;*** FUEE.LSP: [Find UnEqual Elevations] builds a selection set of (contour) 
;*** polylines on various layers with names frequently used for contours.
;*** It first searches for polylines having vertices that have different Z-
;*** values than the first vertex in the polyline; if found, a prompt is issued 
;*** for the correct Z-value and every vertex in the p-line is changed to that 
;*** value.  All other polylines are inspected to insure that their Z-values 
;*** are within 0.00000001 of an integer value; if not, they are rounded to the
;*** nearest foot.
;***
;*** By: Randy Benson
;***     W. R. Benson & Associates, Inc.
;***     (818) 897-1254
;***
;*** FUEE.LSP  v.1.01
;*** March 1, 1994
;***
(princ " Loading FUEE.LSP: Rounds contours to nearest foot & corrects contours")
(princ "\n          with differing vertex Z-values (vertex elevations)... ")

(defun err (msg)
  (if (not (member msg '("console break" "Function cancelled")))
    (princ (strcat "\nError: " msg "\n"))
    (princ (strcat "\nError: " "==" msg "\n"))
  )
  (if olderr (setq *error* olderr))
  (setvar   "CMDECHO" oldcmd)
  (setq args '(cnt cntr count df ed ef ef1 el emax en ev lst n new10 oldelv
               p ss ss1 vn vrtx args))
  (foreach arg args (set arg nil));......clean up locals conv'd to globals
  (command nil nil nil);.................& back out
  (command "_.UNDO" "_E");...............gracefully
  (princ)
)

(defun c:fuee () ; ( / cnt cntr count df ed ef ef1 el emax en ev lst n new10 oldelv
;                  p vn vrtx)
  (setq olderr *error* 
        *error* err
        oldcmd (getvar   "CMDECHO")
  )
  (setvar   "CMDECHO" 0)
  (princ "\nBuilding selection set...\n")
  (setq ss
    (ssget "X"
     '(
        (-4 . "<or")
          (-4 . "<and")
            (0 . "POLYLINE") (8 . "*CONTOUR*")
          (-4 . "and>")
          (-4 . "<and")
            (0 . "POLYLINE") (8 . "IND-*")
          (-4 . "and>")
          (-4 . "<and")
            (0 . "POLYLINE") (8 . "INT-*")
          (-4 . "and>")
        (-4 . "or>")
      )
    )
        count 0                ; counts  ss -  all contour plines
          cnt 0                ; counts ss1 - plines w/ unequal vert's
         cntr 0                ; counts ss2 - plines w/ fractional elev's
            n 0                ; counts spinning symbol 
         emax (sslength ss)    ; end of ss
          ss1 (ssadd) 
          ss2 (ssadd)
  )
 (princ "       Searching selection set...\n")
  (while (< count emax)
    (setq en (ssname ss count)          
          ed (entget en)
        vrtx (entnext (cdr (assoc -1 ed)))
          vn 0
          el (caddr (cdr (assoc 10 (entget vrtx))))
          ef 1
          df 0
         lst (list '"|" '"/" '"-" '"\\")
    )
    (princ "\010")                                ; backspace
    (while (/= "SEQEND" (cdr (assoc 0 (setq ev (entget vrtx)))))
      (setq vn (1+ vn))
      (if (/= el (caddr (cdr (assoc 10 ev))))
        (progn
          (ssadd en ss1)
          (if ef
            (progn
              (princ (strcat "Vertex no. 1 elev = " 
                                           (rtos el 2 2) "\n"))
              (setq ef nil
                   ef1 1
              )
            )
          )
          (princ (strcat "Vertex no. " (rtos vn 2 0) " elev = "
                 (rtos (caddr (cdr (assoc 10 ev))) 2 2) "\n"))
          (setq n 5
               df (1+ df)
          )
        )
      )
      (setq vrtx (entnext vrtx))
    )
    (if ef1
      (progn
        (setq ef1 nil)
        (princ (strcat (rtos df 2 0) " unequal vertices out of "
                       (rtos vn 2 0) " total vertices in contour\n"))
        (setq ell
          (getreal (strcat "Elevation to use <" (rtos el) ">: "))
        )
        (if (null ell)
          () (setq el ell)
        )
        (setq vn 0
            vrtx (entnext (cdr (assoc -1 ed)))
        )
        (while (/= "SEQEND" (cdr (assoc 0 (setq ev (entget vrtx)))))
          (setq vn (1+ vn)
                ev (entget vrtx)
            oldelv (caddr (cdr (assoc 10 ev)))
          )
          (setq new10 (subst el oldelv (assoc 10 ev)))
          (setq ev (subst new10 (assoc 10 ev) ev))
          (entmod ev)
          (setq vrtx (entnext vrtx))
        )
        (entupd en)
      )
      (progn
        (if (< 0.00000001 (abs (- el (atof (rtos el 2 0)))))
          (progn
            (setq el (atof (rtos el 2 0))
                cntr (1+ cntr)
                 ss2 (ssadd en ss2)
            )
            (command "_.CHANGE" en "" "_P" "_E" el "_T" 0.0 "")
          )
        )
      )
    )
    (setq count (1+ count))
    (if (> n 4) 
      (progn 
        (setq n 0)
        (princ "\n")
      )
      (progn
        (setq p (strcat "\010" (nth n lst)))
        (setq n (1+ n))
        (princ p)
      )
    )
    (if (> n 3) (setq n 0))
  ); ----------------------------- end while < count emax
  (if ss1 
    (setq cnt (sslength ss1))
  )
  (princ (strcat "\010Done.  Searched " (itoa count) " polylines.
       Found and fixed "      (itoa cnt)
                 " contour line(s) with unequal vertex elevations.
       Found and fixed "      (itoa cntr)
                 " contour line(s) with fractional elevations.\n"))
  (princ "                    Selection set of all polylines: ss\n")
  (princ "       Selection set of polylines w/ unequal elevs: ss1\n")
  (princ "    Selection set of polylines w/ fractional elevs: ss2\n")
  (initget 0 "Yes No")
  (setq cont (getkword "\nRelease selection sets from memory? (Y/N) <N>: "))
    (if (equal cont "Yes")
      (progn
        (setq ss nil
             ss1 nil
             ss2 nil
        )
      )
    )
  (setq *error* olderr
        olderr nil
          cont nil
  )
  (setvar   "CMDECHO" oldcmd)
  (princ)
)
(princ "Loaded")
(princ)
